@@ -19,22 +19,26 @@
  */
 package com.eteks.sweethome3d.swing;
 
+import java.awt.BasicStroke;
 import java.awt.Color;
 import java.awt.Component;
 import java.awt.Container;
 import java.awt.Cursor;
 import java.awt.Dimension;
 import java.awt.EventQueue;
+import java.awt.Font;
 import java.awt.Frame;
 import java.awt.Graphics;
 import java.awt.Graphics2D;
 import java.awt.GraphicsDevice;
 import java.awt.GraphicsEnvironment;
 import java.awt.GridBagLayout;
+import java.awt.Image;
 import java.awt.Insets;
 import java.awt.MouseInfo;
 import java.awt.Point;
 import java.awt.Rectangle;
+import java.awt.Stroke;
 import java.awt.Toolkit;
 import java.awt.Window;
 import java.awt.event.ActionEvent;
@@ -58,6 +62,7 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Enumeration;
 import java.util.HashMap;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
@@ -66,11 +71,14 @@ import java.util.ResourceBundle;
 import java.util.concurrent.Executors;
 
 import javax.imageio.ImageIO;
+import javax.imageio.ImageReader;
+import javax.imageio.stream.ImageInputStream;
 import javax.jnlp.BasicService;
 import javax.jnlp.ServiceManager;
 import javax.jnlp.UnavailableServiceException;
 import javax.swing.BorderFactory;
 import javax.swing.ButtonGroup;
+import javax.swing.ImageIcon;
 import javax.swing.JComponent;
 import javax.swing.JDialog;
 import javax.swing.JMenu;
@@ -94,6 +102,8 @@ import javax.swing.event.PopupMenuEvent;
 import javax.swing.event.PopupMenuListener;
 import javax.swing.text.JTextComponent;
 
+import com.eteks.sweethome3d.model.Content;
+import com.eteks.sweethome3d.model.Polyline;
 import com.eteks.sweethome3d.model.TextureImage;
 import com.eteks.sweethome3d.model.UserPreferences;
 import com.eteks.sweethome3d.tools.OperatingSystem;
@@ -342,7 +352,8 @@ public class SwingTools {
     if (OperatingSystem.isMacOSX()
         && (language.equals(Locale.CHINESE.getLanguage())
             || language.equals(Locale.JAPANESE.getLanguage())
-            || language.equals(Locale.KOREAN.getLanguage()))) {
+            || language.equals(Locale.KOREAN.getLanguage())
+            || language.equals("uk"))) {  // Ukrainian
       int openingBracketIndex = localizedString.indexOf('(');
       if (openingBracketIndex != -1) {
         int closingBracketIndex = localizedString.indexOf(')');
@@ -923,4 +934,183 @@ public class SwingTools {
       }
     }
   }
+
+  /**
+   * Returns <code>image</code> size in pixels.
+   * @return the size or <code>null</code> if the information isn't given in the meta data of the image
+   */
+  public static Dimension getImageSizeInPixels(Content image) throws IOException {
+    InputStream in = null;
+    try {
+      in = image.openStream();
+      ImageInputStream iis = ImageIO.createImageInputStream(in);
+      Iterator<ImageReader> it = ImageIO.getImageReaders(iis);
+      if (it.hasNext()) {
+        ImageReader reader = (ImageReader)it.next();
+        reader.setInput(iis);
+        int imageWidth = reader.getWidth(reader.getMinIndex());
+        int imageHeight = reader.getHeight(reader.getMinIndex());
+        reader.dispose();
+        return new Dimension(imageWidth, imageHeight);
+      }
+      return null;
+    } finally {
+      if (in != null) {
+        in.close();
+      }
+    }
+  }
+  
+  /**
+   * Returns the line stroke matching the given line styles.
+   */
+  public static Stroke getStroke(float thickness, 
+                                 Polyline.CapStyle capStyle,
+                                 Polyline.JoinStyle joinStyle, 
+                                 Polyline.DashStyle dashStyle) {
+    int strokeCapStyle;
+    switch (capStyle) {
+      case ROUND :
+        strokeCapStyle = BasicStroke.CAP_ROUND;
+        break;
+      case SQUARE :
+        strokeCapStyle = BasicStroke.CAP_SQUARE;
+        break;
+      default:
+        strokeCapStyle = BasicStroke.CAP_BUTT;
+        break;
+    }
+    
+    int strokeJoinStyle;
+    switch (joinStyle) {
+      case ROUND :
+      case CURVED :
+        strokeJoinStyle = BasicStroke.JOIN_ROUND;
+        break;
+      case BEVEL :
+        strokeJoinStyle = BasicStroke.JOIN_BEVEL;
+        break;
+      default:
+        strokeJoinStyle = BasicStroke.JOIN_MITER;
+        break;
+    }
+    
+    float [] strokeDashes;
+    switch (dashStyle) {
+      case DOT :
+        strokeDashes = new float [] {thickness, thickness}; 
+        break;
+      case DASH :
+        strokeDashes = new float [] {thickness * 4, thickness * 2}; 
+        break;
+      case DASH_DOT :
+        strokeDashes = new float [] {thickness * 8, thickness * 2, thickness * 2, thickness * 2};
+        break;
+      case DASH_DOT_DOT :
+        strokeDashes = new float [] {thickness * 8, thickness * 2, thickness * 2, thickness * 2, thickness * 2, thickness * 2};
+        break;
+      default :
+        strokeDashes = null;
+        break;
+    }
+    
+    return new BasicStroke(thickness, strokeCapStyle, strokeJoinStyle, 10, strokeDashes, 0);
+  }
+
+  /**
+   * Updates Swing components default size according to resolution scale.
+   */
+  static void updateComponentDefaults() {
+    float resolutionScale = getResolutionScale();
+    if (resolutionScale != 1) {
+      Font buttonFont = updateComponentFontSize("Button.font", resolutionScale);
+      updateComponentFontSize("ToggleButton.font", resolutionScale);
+      updateComponentFontSize("RadioButton.font", resolutionScale);
+      updateComponentFontSize("CheckBox.font", resolutionScale);
+      updateComponentFontSize("ColorChooser.font", resolutionScale);
+      updateComponentFontSize("ComboBox.font", resolutionScale);
+      updateComponentFontSize("InternalFrame.titleFont", resolutionScale);
+      Font labelFont = updateComponentFontSize("Label.font", resolutionScale);
+      updateComponentFontSize("List.font", resolutionScale);
+      updateComponentFontSize("MenuBar.font", resolutionScale);
+      updateComponentFontSize("MenuItem.font", resolutionScale);
+      updateComponentFontSize("MenuItem.acceleratorFont", resolutionScale);
+      updateComponentFontSize("RadioButtonMenuItem.font", resolutionScale);
+      updateComponentFontSize("RadioButtonMenuItem.acceleratorFont", resolutionScale);
+      updateComponentFontSize("CheckBoxMenuItem.font", resolutionScale);
+      updateComponentFontSize("CheckBoxMenuItem.acceleratorFont", resolutionScale);
+      updateComponentFontSize("Menu.font", resolutionScale);
+      updateComponentFontSize("Menu.acceleratorFont", resolutionScale);
+      updateComponentFontSize("PopupMenu.font", resolutionScale);
+      updateComponentFontSize("OptionPane.font", resolutionScale);
+      updateComponentFontSize("Panel.font", resolutionScale);
+      updateComponentFontSize("ProgressBar.font", resolutionScale);
+      updateComponentFontSize("ScrollPane.font", resolutionScale);
+      updateComponentFontSize("Viewport.font", resolutionScale);
+      updateComponentFontSize("Slider.font", resolutionScale);
+      updateComponentFontSize("Spinner.font", resolutionScale);
+      updateComponentFontSize("Table.font", resolutionScale);
+      updateComponentFontSize("TabbedPane.font", resolutionScale);
+      updateComponentFontSize("TableHeader.font", resolutionScale);
+      updateComponentFontSize("TextField.font", resolutionScale);
+      updateComponentFontSize("FormattedTextField.font", resolutionScale);
+      updateComponentFontSize("PasswordField.font", resolutionScale);
+      updateComponentFontSize("TextArea.font", resolutionScale);
+      updateComponentFontSize("EditorPane.font", resolutionScale);
+      updateComponentFontSize("TitledBorder.font", resolutionScale);
+      updateComponentFontSize("ToolBar.font", resolutionScale);
+      updateComponentFontSize("ToolTip.font", resolutionScale);
+      updateComponentFontSize("Tree.font", resolutionScale);
+      UIManager.put("OptionPane.messageFont", labelFont);
+      UIManager.put("OptionPane.buttonFont", buttonFont);
+      int dividerSize = UIManager.getInt("SplitPane.dividerSize");
+      if (dividerSize != 0) {
+        UIManager.put("SplitPane.dividerSize", Math.round(dividerSize * resolutionScale));
+      }
+    }
+  }
+
+  private static Font updateComponentFontSize(String fontKey, float resolutionScale) {
+    Font font = UIManager.getFont(fontKey);
+    if (font != null) {
+      font = font.deriveFont(font.getSize() * resolutionScale);
+      UIManager.put(fontKey, font);
+    }
+    return font;
+  }
+  
+  /**
+   * Returns a scale factor used to adapt user interface items to screen resolution.
+   */
+  public static float getResolutionScale() {
+    String resolutionScaleProperty = System.getProperty("com.eteks.sweethome3d.resolutionScale");
+    if (resolutionScaleProperty != null) {
+      try {
+        return Float.parseFloat(resolutionScaleProperty.trim());
+      } catch (NumberFormatException ex) {
+        // Ignore resolution
+      }
+    }
+    
+    return 1f;
+  }
+  
+  /**
+   * Returns an image icon scaled according to the value returned by {@link #getResolutionScale()}.
+   */
+  public static ImageIcon getScaledImageIcon(URL imageUrl) {
+    float resolutionScale = getResolutionScale();
+    if (resolutionScale == 1) {
+      return new ImageIcon(imageUrl);
+    } else {
+      try {
+        BufferedImage image = ImageIO.read(imageUrl);
+        Image scaledImage = image.getScaledInstance(Math.round(image.getWidth() * resolutionScale), 
+            Math.round(image.getHeight() * resolutionScale), Image.SCALE_SMOOTH);
+        return new ImageIcon(scaledImage);
+      } catch (IOException ex) {
+        return null;
+      }
+    }
+  }
 }
